﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;
using Microsoft.Interop.Security.AzRoles;
using HRServiceBus;

namespace HRService
{

    /// <summary>
    /// Implements logic to get the full list of operation access rights out of the store.
    /// </summary>
    /// <remarks>
    /// This implementation works, but requires opening the store on each call to the service.
    /// A production application should optimize this by caching the initialized store in memory to
    /// improve the performance of the system. The store is not serializable, so it should be stored in
    /// memory on the local machine.
    /// </remarks>
    class Auth_AzMan_Direct : IDisposable, IAuthProvider
    {

        #region Member Variables

        private IAzAuthorizationStore store;

        private IAzApplication3 app;

        private bool disposed = false;

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor - Open the auth store
        /// </summary>
        /// <param name="connectionString">The connection string to the AzMan policy store.</param>
        /// <param name="applicationName">The name of the AzMan application in the policy store.</param>
        public Auth_AzMan_Direct(string connectionString, string applicationName)
        {
            store = new AzAuthorizationStoreClass();
            store.Initialize(0, connectionString, null);

            app = (IAzApplication3)store.OpenApplication(applicationName, null);
        }

        #endregion
        
        #region Public Methods

        /// <summary>
        /// Check the persmissions that are applicable for the user of the context object.
        /// </summary>
        /// <param name="ctx">Context object to update.</param>
        public void ApplyPermissions(Context ctx)
        {
            const int NO_ERROR = 0;

            // Initialize by name.
            // IAzClientContext3 userCtx = (IAzClientContext3)app.InitializeClientContextFromName(ctx.User.Name, "OFFICE", null);

            // Initialize by token. (Most efficient).
            IntPtr token = ((WindowsIdentity) ctx.User).Token;
            ulong hToken = Convert.ToUInt64(token.ToInt64());
            IAzClientContext3 userCtx = (IAzClientContext3)app.InitializeClientContextFromToken(hToken, null);

            // Check all operations up front.
            // You could check these on an as needed basis if you prefer.
            object[] operations = 
            { 
                OperationIds.AddEmployee, 
                OperationIds.GetEmployee,
                OperationIds.GetEmployeeAddress, 
                OperationIds.GetEmployeeBonus ,
                OperationIds.GetEmployeeSalary ,
                OperationIds.GetEmployeeSSN ,
                OperationIds.ListEmployees ,
                OperationIds.ReinstateEmployee ,
                OperationIds.SetEmployeeAddress ,
                OperationIds.SetEmployeeManager ,
                OperationIds.SetEmployeePosition ,
                OperationIds.SetEmployeeSalary ,
                OperationIds.SetEmployeeSSN ,
                OperationIds.TerminateEmployee ,
                OperationIds.UpdateEmployee
            };



            // -------------------------------------------------------------------
            // Get standard rights
            // -------------------------------------------------------------------
            object[] scopes = { "" };

            // Call the AzMan method to check access for the user.
            object[] results = (object[]) userCtx.AccessCheck(
                "HRService Demo", 
                scopes, 
                operations, 
                null, null, null, null, null
            );

            if (null == results || results.Length != 15)
                throw new ApplicationException("Authorization access check did not return the expected number of results.");

            // Pull the authorization (allowed / denied) for each of the requested operations.
            // A value of 0 means "Authorized" or "Allowed". Otherwise, access is denied.
            ctx.Rights = new HRServiceEntities.Rights();

            ctx.Rights.AddEmployee = (NO_ERROR == (int)results[0]);
            ctx.Rights.GetEmployee = (NO_ERROR == (int)results[1]);
            ctx.Rights.GetEmployeeAddress = (NO_ERROR == (int)results[2]);
            ctx.Rights.GetEmployeeBonus = (NO_ERROR == (int)results[3]);
            ctx.Rights.GetEmployeeSalary = (NO_ERROR == (int)results[4]);
            ctx.Rights.GetEmployeeSSN = (NO_ERROR == (int)results[5]);
            ctx.Rights.ListEmployees = (NO_ERROR == (int)results[6]);
            ctx.Rights.ReinstateEmployees = (NO_ERROR == (int)results[7]);
            ctx.Rights.SetEmployeeAddress = (NO_ERROR == (int)results[8]);
            ctx.Rights.SetEmployeeManager = (NO_ERROR == (int)results[9]);
            ctx.Rights.SetEmployeePosition = (NO_ERROR == (int)results[10]);
            ctx.Rights.SetEmployeeSalary = (NO_ERROR == (int)results[11]);
            ctx.Rights.SetEmployeeSSN = (NO_ERROR == (int)results[12]);
            ctx.Rights.TerminateEmployee = (NO_ERROR == (int)results[13]);
            ctx.Rights.UpdateEmployee = (NO_ERROR == (int)results[14]);


            // -------------------------------------------------------------------
            // Get rights for Self
            // -------------------------------------------------------------------
            scopes[0] = "Self";

            // Call the AzMan method to check access for the user.
            results = (object[])userCtx.AccessCheck(
                "HRService Demo",
                scopes,
                operations,
                null, null, null, null, null
            );

            if (null == results || results.Length != 15)
                throw new ApplicationException("Authorization access check did not return the expected number of results.");

            // Pull the authorization (allowed / denied) for each of the requested operations.
            // A value of 0 means "Authorized" or "Allowed". Otherwise, access is denied.
            ctx.RightsToSelf = new HRServiceEntities.Rights();

            ctx.RightsToSelf.AddEmployee = (NO_ERROR == (int)results[0]);
            ctx.RightsToSelf.GetEmployee = (NO_ERROR == (int)results[1]);
            ctx.RightsToSelf.GetEmployeeAddress = (NO_ERROR == (int)results[2]);
            ctx.RightsToSelf.GetEmployeeBonus = (NO_ERROR == (int)results[3]);
            ctx.RightsToSelf.GetEmployeeSalary = (NO_ERROR == (int)results[4]);
            ctx.RightsToSelf.GetEmployeeSSN = (NO_ERROR == (int)results[5]);
            ctx.RightsToSelf.ListEmployees = (NO_ERROR == (int)results[6]);
            ctx.RightsToSelf.ReinstateEmployees = (NO_ERROR == (int)results[7]);
            ctx.RightsToSelf.SetEmployeeAddress = (NO_ERROR == (int)results[8]);
            ctx.RightsToSelf.SetEmployeeManager = (NO_ERROR == (int)results[9]);
            ctx.RightsToSelf.SetEmployeePosition = (NO_ERROR == (int)results[10]);
            ctx.RightsToSelf.SetEmployeeSalary = (NO_ERROR == (int)results[11]);
            ctx.RightsToSelf.SetEmployeeSSN = (NO_ERROR == (int)results[12]);
            ctx.RightsToSelf.TerminateEmployee = (NO_ERROR == (int)results[13]);
            ctx.RightsToSelf.UpdateEmployee = (NO_ERROR == (int)results[14]);

        }

        #endregion
        
        #region IDisposable Members

        public void Dispose()
        {
            if (!disposed)
            {
                if (null != store && null != app)
                {
                    // Release the com object now so that it is not held in memory.
                    int refCount = 1;
                    while (refCount > 0)
                        refCount = System.Runtime.InteropServices.Marshal.ReleaseComObject(app);

                    // Release the com object now so that it is not held in memory.
                    refCount = 1;
                    while (refCount > 0)
                        refCount = System.Runtime.InteropServices.Marshal.ReleaseComObject(store);

                    store = null;
                    app = null;

                    disposed = true;
                }
            }
        }

        #endregion

    }
}
