﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Security.Principal;
using HRServiceBus;
using HRServiceEntities;

namespace HRService
{

    public class HRService : IHRService
    {

        public List<Employee> ListEmployees()
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Do an initial check to see if the user has permission to this method:
            if (!ctx.Rights.ListEmployees)
            {
                throw new ApplicationException(string.Format("{0} does not have permission to {1}.", ctx.User.Name, "list employees"));
            }

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetEmployees(ctx);
        }

        public Employee GetEmployee(string ID)
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Do an initial check to see if the user has permission to this method:
            if (!ctx.Rights.GetEmployee)
            {
                throw new ApplicationException(string.Format("{0} does not have permission to {1}.", ctx.User.Name, "get an employee record"));
            }

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetEmployee(ctx, ID);
        }

        public void UpdateEmployee(EmployeeUpdateInfo UpdatedEmployee)
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Do an initial check to see if the user has permission to this method:
            if (!ctx.Rights.UpdateEmployee)
            {
                throw new ApplicationException(string.Format("{0} does not have permission to {1}.", ctx.User.Name, "update an employee record"));
            }

            // Call the Bus
            HRBus bus = new HRBus();
            bus.UpdateEmployee(ctx, UpdatedEmployee);

            return;
        }

        public Employee GetMyEmployeeRecord()
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Do an initial check to see if the user has permission to this method:
            if (!ctx.RightsToSelf.GetEmployee)
            {
                throw new ApplicationException(string.Format("{0} does not have permission to {1}.", ctx.User.Name, "get an employee record"));
            }

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetMyEmployeeRecord(ctx);
        }

        public void UpdateMyEmployeeRecord(EmployeeUpdateInfo UpdatedEmployee)
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Do an initial check to see if the user has permission to this method:
            if (!ctx.RightsToSelf.UpdateEmployee)
            {
                throw new ApplicationException(string.Format("{0} does not have permission to {1}.", ctx.User.Name, "update an employee record"));
            }

            // Call the Bus
            HRBus bus = new HRBus();
            bus.UpdateMyEmployeeRecord(ctx, UpdatedEmployee);

            return;
        }

        public Rights GetMyRights()
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetRights(ctx);
        }

        public Rights GetMyRightsToSelf()
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext();

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetRightsToSelf(ctx);
        }

        public Rights GetRightsForUser(string UserName)
        {
            // Create and populate the Context class.
            // This will get the identity of the user and then load / populate the 
            // permissions from the currently configured authorization implementation.
            // (See the HRService's app.config to change the auth implementation.)
            Context ctx = this.GetContext(UserName);

            // Call the Bus
            HRBus bus = new HRBus();
            return bus.GetRights(ctx);
        }


        #region Internal Methods

        /// <summary>
        /// Returns a context object based on the callers identity, 
        /// populated with the permissions that they have to the system.
        /// </summary>
        /// <returns>A populated context object.</returns>
        /// <remarks>
        /// Uses the authorization implementation that is defined in the HRService app.config
        /// </remarks>
        private Context GetContext()
        {
            // Pull the identity from WCF.
            IIdentity user = OperationContext.Current.ServiceSecurityContext.PrimaryIdentity;

            Context ctx = new Context(user);

            ApplyPermissions(ctx);

            return ctx;
        }

        /// <remarks>
        /// Uses the authorization implementation that is defined in the HRService app.config
        /// </remarks>
        private Context GetContext(string UserName)
        {
            IIdentity user = new WindowsIdentity(UserName);

            Context ctx = new Context(user);

            ApplyPermissions(ctx);

            return ctx;
        }

        /// <summary>
        /// Determines the configured authorization implementation based on the app.config. 
        /// Then populates the Context accordingly.
        /// </summary>
        /// <param name="ctx"></param>
        private void ApplyPermissions(Context ctx)
        {
            switch (Properties.Settings.Default.AuthMode.ToUpper())
            {

                case "DIRECT":
                    // Get the permissions that this user has.
                    using (IAuthProvider auth = new Auth_AzMan_Direct(Properties.Settings.Default.AuthStore, Properties.Settings.Default.AuthStoreAppName))
                    {
                        auth.ApplyPermissions(ctx);
                    }

                    break;

                case "ENTLIB":
                    // Get the permissions that this user has.
                    using (IAuthProvider auth = new Auth_AzMan_EntLib())
                    {
                        auth.ApplyPermissions(ctx);
                    }

                    break;

                case "ENTLIBRULES":
                    // Get the permissions that this user has.
                    using (IAuthProvider auth = new Auth_EntLib_Rules())
                    {
                        auth.ApplyPermissions(ctx);
                    }

                    break;

                default:
                    // Get the permissions that this user has.
                    using (IAuthProvider auth = new Auth_NoAzMan())
                    {
                        auth.ApplyPermissions(ctx);
                    }

                    break;
            }
        }

        #endregion


    }
}
